/*
 * The Clear BSD License
 * Copyright (c) 2017, NXP Semiconductors, Inc.
* All rights reserved.
*
 * 
* Redistribution and use in source and binary forms, with or without modification,
* are permitted (subject to the limitations in the disclaimer below) provided
*  that the following conditions are met:
*
* o Redistributions of source code must retain the above copyright notice, this list
*   of conditions and the following disclaimer.
*
* o Redistributions in binary form must reproduce the above copyright notice, this
*   list of conditions and the following disclaimer in the documentation and/or
*   other materials provided with the distribution.
*
* o Neither the name of the copyright holder nor the names of its
*   contributors may be used to endorse or promote products derived from this
*   software without specific prior written permission.
*
* NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "notifications_driver.h"
#include "string.h"

static uint8_t notification_sms_unread = 0;
static uint8_t notification_email_unread = 0;
static uint8_t notification_call_unread = 0;

/* Sets the notifications' counter's value.*/
void Notification_SetUnreadCounter(notification_type_t notifType, uint8_t value)
{
    if (value > 0)
    {
        switch (notifType)
        {
            case NOTIF_TYPE_CALL:
            {
                notification_call_unread = value;
                break;
            }
            case NOTIF_TYPE_MAIL:
            {
                notification_email_unread = value;
                break;
            }
            case NOTIF_TYPE_SMS:
            {
                notification_sms_unread = value;
                break;
            }
        }
    }
}

/* Gets the notifications' counter's value.*/
uint8_t Notification_GetUnreadCounter(notification_type_t notifType)
{
    switch (notifType)
    {
        case NOTIF_TYPE_CALL:
        {
            return notification_call_unread;
        }
        case NOTIF_TYPE_MAIL:
        {
            return notification_email_unread;
        }
        case NOTIF_TYPE_SMS:
        {
            return notification_sms_unread;
        }
    }
    return 0xFF;
}
