/*
 * The Clear BSD License
 * Copyright (c) 2016, NXP Semiconductors, Inc.
* All rights reserved.
*
 * 
* Redistribution and use in source and binary forms, with or without modification,
* are permitted (subject to the limitations in the disclaimer below) provided
*  that the following conditions are met:
*
* o Redistributions of source code must retain the above copyright notice, this list
*   of conditions and the following disclaimer.
*
* o Redistributions in binary form must reproduce the above copyright notice, this
*   list of conditions and the following disclaimer in the documentation and/or
*   other materials provided with the distribution.
*
* o Neither the name of the copyright holder nor the names of its
*   contributors may be used to endorse or promote products derived from this
*   software without specific prior written permission.
*
* NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "ftm_buzzer.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*******************************************************************************
 * Variables
 ******************************************************************************/

extern volatile touch_type_t pressedTouch;

static const uint16_t scales[3][12] = {{523, 554, 587, 622, 659, 698, 740, 784, 831, 880, 932, 988},
                                       {1046, 1108, 1174, 1244, 1318, 1397, 1480, 1568, 1662, 1760, 1865, 1975},
                                       {2093, 2217, 2349, 2489, 2637, 2794, 2960, 3136, 3322, 3520, 3729, 3951}};

/*******************************************************************************
 * Code
 ******************************************************************************/
/* Initizlize the ftm */
void BUZZER_FTM_Init(void)
{
    ftm_config_t ftmInfo;
    ftm_chnl_pwm_signal_param_t ftmParam;
    FTM_GetDefaultConfig(&ftmInfo);
    ftmParam.chnlNumber = BOARD_BUZZER_FTM_CHANNEL;
    ftmParam.level = kFTM_LowTrue;
    ftmParam.dutyCyclePercent = 50U;
    ftmParam.firstEdgeDelayPercent = 0U;
    ftmInfo.pwmSyncMode = kFTM_SoftwareTrigger;
    /* Initialize FTM module */
    FTM_Init(BOARD_BUZZER_FTM_BASEADDR, &ftmInfo);
    FTM_SetupPwm(BOARD_BUZZER_FTM_BASEADDR, &ftmParam, 1U, kFTM_EdgeAlignedPwm, 740, BOARD_BUZZER_FTM_SOURCE_CLOCK);
}
/* Play sound */
void PlaySound(uint16_t freqInHz, uint16_t durationInMs)
{
    if (touch_invalid == pressedTouch)
    {
        FTM_StopTimer(BOARD_BUZZER_FTM_BASEADDR);
    }
    else
    {
        FTM_StartTimer(BOARD_BUZZER_FTM_BASEADDR, kFTM_SystemClock);
        ftm_chnl_pwm_signal_param_t ftmParam;
        ftmParam.chnlNumber = BOARD_BUZZER_FTM_CHANNEL;
        ftmParam.level = kFTM_LowTrue;
        ftmParam.dutyCyclePercent = 50U;
        ftmParam.firstEdgeDelayPercent = 0U;
        FTM_SetupPwm(BOARD_BUZZER_FTM_BASEADDR, &ftmParam, 1U, kFTM_CenterAlignedPwm, freqInHz,
                     BOARD_BUZZER_FTM_SOURCE_CLOCK);
        FTM_SetSoftwareTrigger(BOARD_BUZZER_FTM_BASEADDR, true);
        Timer_Delay_ms(durationInMs);
        FTM_StopTimer(BOARD_BUZZER_FTM_BASEADDR);
    }
}

/* Play the sound ode to joy */
void OdeToJoy(void)
{
    PlaySound(scales[0][7], QUARTER_NOTE);     /* < G */
    PlaySound(scales[0][7], QUARTER_NOTE);     /* < G */
    PlaySound(scales[0][8], QUARTER_NOTE);     /* < A */
    PlaySound(scales[0][10], QUARTER_NOTE);    /* < B */
    PlaySound(scales[0][10], QUARTER_NOTE);    /* < B */
    PlaySound(scales[0][8], QUARTER_NOTE);     /* < A */
    PlaySound(scales[0][7], QUARTER_NOTE);     /* < G */
    PlaySound(scales[0][5], QUARTER_NOTE);     /* < F */
    PlaySound(scales[0][3], QUARTER_NOTE);     /* < D# */
    PlaySound(scales[0][3], QUARTER_NOTE);     /* < E */
    PlaySound(scales[0][5], QUARTER_NOTE);     /* < F */
    PlaySound(scales[0][7], QUARTER_NOTE);     /* < G */
    PlaySound(scales[0][7], QUARTER_DOT_NOTE); /* < G. */
    PlaySound(scales[0][5], EIGHTH_NOTE);      /* < F */
    PlaySound(scales[0][5], QUARTER_DOT_NOTE); /* < F. */
}
