/**
 * \file heartRate_driver.h
 * \version 1.00
 * \brief this file contains heart-rate functionality
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * Neither the name of NXP, nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * visit: http://www.mikroe.com and http://www.nxp.com
 *
 * get support at: http://www.mikroe.com/forum and https://community.nxp.com
 *
 * Project HEXIWEAR, 2015
 */

#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "fsl_common.h"
#include "gui_driver.h"

#define HR_SAMPLE_RATE (100) /** sample rate for MAXIM 30101 sensor */

extern volatile bool g_heartUpdated;
extern guiLabel_t gui_heartRate_bpmVal_label;
extern guiLabel_t gui_heartRate_bpmText_label;

extern guiImage_t gui_heartRate_icon;
extern guiImage_t screen_buttonStop;
extern guiImage_t screen_buttonStart;
/** HR global status enum */
typedef enum {
    HEART_RATE_STATUS_SUCCESS,
    HEART_RATE_STATUS_ERROR

} heartRate_status_t;

/**
 * register heart-rate sensor and FFT
 */
void HeartRate_Init(void);

/**
 * unregister from HR sensor and free used memory
 */
void HeartRate_Deinit(void);

/**
 * reallocate the buffers and reset the relevant variables
 */
status_t HeartRate_Resume(void);

/**
 * free used memory and unregister from HR sensor
 */
void HeartRate_Pause(void);

/**
 * process available HR data
 * @param sampleBuf   new data buffer
 * @param numAvailSam number of available data values
 * @param param       optional parameter
 */
void HeartRate_Process(void *sampleBuf, uint8_t numAvailSam, void *param);

/**
 * Update heartrate bmp value in GUI.
 */
void HeartRate_UpateGUI(void);

/**
 * Resume heartrate bmp screen in GUI.
 */
void HeartRate_ResumeGUI(void);

/**
 * Pause heartrate bmp screen and recover the heartrate icon screen in GUI.
 */
void HeartRate_PauseGUI(void);
