/*
 * The Clear BSD License
 * Copyright (c) 2016, NXP Semiconductors, Inc.
 * All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 * that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _BOARD_H_
#define _BOARD_H_

#include "clock_config.h"
#include "fsl_gpio.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/* The board name */
#define BOARD_NAME "HEXIWEAR"

/* The UART to use for debug messages. */
#define BOARD_DEBUG_UART_TYPE DEBUG_CONSOLE_DEVICE_TYPE_UART
#define BOARD_DEBUG_UART_BASEADDR (uint32_t) UART0
#define BOARD_DEBUG_UART_INSTANCE 0U
#define BOARD_DEBUG_UART_CLKSRC SYS_CLK
#define BOARD_DEBUG_UART_CLK_FREQ CLOCK_GetCoreSysClkFreq()
#define BOARD_UART_IRQ UART0_RX_TX_IRQn
#define BOARD_UART_IRQ_HANDLER UART0_RX_TX_IRQHandler

#ifndef BOARD_DEBUG_UART_BAUDRATE
#define BOARD_DEBUG_UART_BAUDRATE 115200
#endif /* BOARD_DEBUG_UART_BAUDRATE */

/* Define the port for oled transfer */
#ifndef BOARD_OLED_CS_GPIO
#define BOARD_OLED_CS_GPIO GPIOB
#endif
#ifndef BOARD_OLED_RST_GPIO
#define BOARD_OLED_RST_GPIO GPIOE
#endif
#ifndef BOARD_OLED_DC_GPIO
#define BOARD_OLED_DC_GPIO GPIOD
#endif
#ifndef BOARD_OLED_PWR_GPIO
#define BOARD_OLED_PWR_GPIO GPIOC
#endif

#ifndef BOARD_OLED_CS_PIN
#define BOARD_OLED_CS_PIN 20U
#endif
#ifndef BOARD_OLED_RST_PIN
#define BOARD_OLED_RST_PIN 6U
#endif
#ifndef BOARD_OLED_DC_PIN
#define BOARD_OLED_DC_PIN 15U
#endif
#ifndef BOARD_OLED_PWR_PIN
#define BOARD_OLED_PWR_PIN 13U
#endif

/* Define DSPI for oled */
#define BOARD_OLED_DSPI_MASTER_BASEADDR DSPI2
#define BOARD_OLED_DSPI_MASTER_TRANSFER_BAUDRATE 8000000
#define BOARD_OLED_DSPI_MASTER_CLK_FREQ CLOCK_GetFreq(DSPI0_CLK_SRC)

/* The UART for KW40 transfer */
#define BOARD_KW40_UART_CLK_FREQ CLOCK_GetFreq(UART4_CLK_SRC)
#define BOARD_KW40_UART_BASEADDR UART4
#define BOARD_KW40_UART_TRANSFER_BAUDRATE 230400

/* The lptmr for delay */
#define BOARD_DELAY_LPTMR_BASEADDR LPTMR0
#define BOARD_DELAY_LPTMR_IRQn LPTMR0_IRQn
#define BOARD_DELAY_LPTMR_IRQHandler LPTMR0_IRQHandler
#define BOARD_DELAY_LPTMR_CLK_FREQ CLOCK_GetFreq(kCLOCK_LpoClk)

/* The ftm for trigger buzzer */
#define BOARD_BUZZER_FTM_BASEADDR FTM0
#define BOARD_BUZZER_FTM_CHANNEL kFTM_Chnl_1
#define BOARD_BUZZER_FTM_SOURCE_CLOCK CLOCK_GetFreq(kCLOCK_BusClk)

/* The i2c for irthermo read temperature */
#define BOARD_IRTHERMO_I2C_MASTER_BASEADDR I2C0
#define BOARD_IRTHERMO_I2C_MASTER_CLK_FREQ CLOCK_GetFreq(I2C0_CLK_SRC)
#define BOARD_IRTHERMO_I2C_MASTER_SLAVE_ADDR_7BIT 0x5A

/* Port for motion */
#define BOARD_MOTION_GPIO GPIOB
#define BOARD_MOTION_PORT PORTB
#define BOARD_MOTION_GPIO_PIN 8U
#define BOARD_MOTION_PORT_IRQn PORTB_IRQn
#define BOARD_MOTION_IRQ_HANDLER PORTB_IRQHandler
#define BOARD_MOTION_ENABLE_GPIO GPIOB
#define BOARD_MOTION_ENABLE_GPIO_PIN 19U

/* Port for relay */
#define BOARD_RELAY1_GPIO GPIOC
#define BOARD_RELAY1_PIN 4U
#define BOARD_RELAY2_GPIO GPIOA
#define BOARD_RELAY2_PIN 10U

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/*******************************************************************************
 * API
 ******************************************************************************/
void BOARD_InitDebugConsole(void);

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* _BOARD_H_ */
